@@ -23,6 +23,7 @@ except ImportError:
     ksc_auth = None
 
 from os_client_config import cloud_config
+from os_client_config import defaults
 from os_client_config import exceptions
 from os_client_config import vendors
 
@@ -41,18 +42,23 @@ VENDOR_FILES = [
     os.path.join(d, 'clouds-public.yaml') for d in VENDOR_SEARCH_PATH]
 
 
+def set_default(key, value):
+    defaults._defaults[key] = value
+
+
 def get_boolean(value):
     if value.lower() == 'true':
         return True
     return False
 
 
-def _get_os_environ(defaults):
+def _get_os_environ():
+    ret = dict(defaults._defaults)
     for (k, v) in os.environ.items():
         if k.startswith('OS_'):
             newkey = k[3:].lower()
-            defaults[newkey] = v
-    return defaults
+            ret[newkey] = v
+    return ret
 
 
 def _auth_update(old_dict, new_dict):
@@ -74,11 +80,7 @@ class OpenStackConfig(object):
         self._config_files = config_files or CONFIG_FILES
         self._vendor_files = vendor_files or VENDOR_FILES
 
-        defaults = dict(
-            auth_plugin='password',
-            compute_api_version='1.1',
-        )
-        self.defaults = _get_os_environ(defaults)
+        self.defaults = _get_os_environ()
 
         # use a config file if it exists where expected
         self.cloud_config = self._load_config_file()
@@ -164,20 +166,50 @@ class OpenStackConfig(object):
                     # Can't find the requested vendor config, go about business
                     pass
 
+        if 'auth' not in cloud:
+            cloud['auth'] = dict()
+
         _auth_update(cloud, our_cloud)
         if 'cloud' in cloud:
             del cloud['cloud']
 
-        return self._fix_project_madness(cloud)
+        return self._fix_backwards_madness(cloud)
 
-    def _fix_project_madness(self, cloud):
-        project_name = None
-        # Do the list backwards so that project_name is the ultimate winner
-        for key in ('tenant_id', 'project_id', 'tenant_name', 'project_name'):
-            if key in cloud:
-                project_name = cloud[key]
-                del cloud[key]
-        cloud['project_name'] = project_name
+    def _fix_backwards_madness(self, cloud):
+        cloud = self._fix_backwards_project(cloud)
+        cloud = self._fix_backwards_auth_plugin(cloud)
+        return cloud
+
+    def _fix_backwards_project(self, cloud):
+        # Do the lists backwards so that project_name is the ultimate winner
+        mappings = {
+            'project_name': ('tenant_id', 'project_id',
+                             'tenant_name', 'project_name'),
+        }
+        for target_key, possible_values in mappings.items():
+            target = None
+            for key in possible_values:
+                if key in cloud:
+                    target = cloud[key]
+                    del cloud[key]
+                if key in cloud['auth']:
+                    target = cloud['auth'][key]
+                    del cloud['auth'][key]
+            cloud['auth'][target_key] = target
+        return cloud
+
+    def _fix_backwards_auth_plugin(self, cloud):
+        # Do the lists backwards so that auth_type is the ultimate winner
+        mappings = {
+            'auth_type': ('auth_plugin', 'auth_type'),
+        }
+        for target_key, possible_values in mappings.items():
+            target = None
+            for key in possible_values:
+                if key in cloud:
+                    target = cloud[key]
+                    del cloud[key]
+            cloud[target_key] = target
         return cloud
 
     def get_all_clouds(self):
@@ -231,7 +263,7 @@ class OpenStackConfig(object):
     def _validate_auth(self, config):
         # May throw a keystoneclient.exceptions.NoMatchingPlugin
         plugin_options = ksc_auth.get_plugin_class(
-            config['auth_plugin']).get_options()
+            config['auth_type']).get_options()
 
         for p_opt in plugin_options:
             # if it's in config.auth, win, kill it from config dict
@@ -252,7 +284,7 @@ class OpenStackConfig(object):
                     ' or environment variables. Missing value {auth_key}'
                     ' required for auth plugin {plugin}'.format(
                         cloud=cloud, files=','.join(self._config_files),
-                        auth_key=p_opt.name, plugin=config['auth_plugin']))
+                        auth_key=p_opt.name, plugin=config.get('auth_type')))
 
             # Clean up after ourselves
             for opt in [p_opt.name] + [o.name for o in p_opt.deprecated_opts]:
@@ -293,8 +325,6 @@ class OpenStackConfig(object):
             args['region_name'] = self._get_region(cloud)
 
         config = self._get_base_cloud_config(cloud)
-        if 'auth' not in config:
-            config['auth'] = dict()
 
         # Can't just do update, because None values take over
         for (key, val) in iter(args.items()):
@@ -306,8 +336,8 @@ class OpenStackConfig(object):
                 if type(config[key]) is not bool:
                     config[key] = get_boolean(config[key])
 
-        if 'auth_plugin' in config:
-            if config['auth_plugin'] in ('', 'None', None):
+        if 'auth_type' in config:
+            if config['auth_type'] in ('', 'None', None):
                 validate = False
 
         if validate and ksc_auth:
